<?php
/*
 * Copyright (c) 2013, MEDIASTUTTGART, http://mediastuttgart.de
 *
 * This file is released under the terms of the MIT license.
 * You can find the complete text in the attached LICENSE
 * file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

class sly_Form_CKEditor extends sly_Form_Textarea {
	protected $fullWidth     = false;
	protected $styleset      = 'default';
	protected $config        = 'default';
	protected $config_values = array();
	protected $clickToLoad   = false;

	/**
	 * @param string $name
	 * @param string $label
	 * @param array  $value
	 * @param string $id
	 */
	public function __construct($name, $label, $value, $id = null, $clickToLoad = false) {
		parent::__construct($name, $label, $value, $id);

		$this->clickToLoad = !!$clickToLoad;

		if (version_compare(sly_Core::getVersion('X.Y'), '0.9', '<')) {
			$baseUrl = sly_Util_HTTP::getBaseUrl(true).'/';
			$assets  = sly_Util_AddOn::assetBaseUri('mediastuttgart/ckeditor');
		}
		else {
			$baseUrl = sly_Core::getContainer()->get('sly-request')->getBaseUrl(true).'/';

			// do not use the class name directly so this file is still PHP 5.2 compatible
			$callback = array('sly\Assets\Util', 'addOnUri');
			$assets   = call_user_func($callback, 'mediastuttgart/ckeditor', '');
		}

		$this->addClass('sly-form-ckeditor');
		$this->addOuterClass('sly-form-ckeditor-row');
		$this->setAttribute('data-baseurl', $baseUrl);
		$this->setAttribute('data-assets', $assets);
	}

	/**
	 * @param  boolean $on
	 * @return sly_Form_CKEditor
	 */
	public function useFullWidth($on = true) {
		(boolean) $on ? $this->addOuterClass('full-width') : $this->removeOuterClass('full-width');
		return $this;
	}

	/**
	 * @param  boolean $on
	 * @return sly_Form_CKEditor
	 */
	public function clickToLoad($on = true) {
		$this->clickToLoad = !!$on;
		return $this;
	}

	/**
	 * @param  string $styleset
	 * @return sly_Form_CKEditor
	 */
	public function setStyleset($styleset) {
		$this->styleset = $styleset;
		return $this;
	}

	/**
	 * @param  string $config
	 * @return sly_Form_CKEditor
	 */
	public function setConfig($config) {
		$this->config = $config;
		return $this;
	}

	/**
	 * @param  string $key
	 * @param  mixed  $value
	 * @return sly_Form_CKEditor
	 */
	public function setConfigValue($key, $value) {
		$this->config_values[$key] = $value;
		return $this;
	}

	/**
	 * @return string
	 */
	public function render() {
		$config = sly_Core::config();
		$styles = $config->get('ckeditor/styleset/'.$this->styleset, array());
		$exts   = $config->get('ckeditor/external_plugins', array());
		$config = array_merge($config->get('ckeditor/config/'.$this->config, array()), $this->config_values);

		$this->setAttribute('data-styles', json_encode($this->applyCallbacks($styles)));
		$this->setAttribute('data-config', json_encode($this->applyCallbacks($config)));
		$this->setAttribute('data-external', json_encode($this->applyCallbacks($exts)));

		$layout = sly_Core::getLayout();
		$me     = 'mediastuttgart/ckeditor';

		if (version_compare(sly_Core::getVersion('X.Y'), '0.9', '<')) {
			$base = sly_Util_AddOn::assetBaseUri($me);

			$layout->addCSSFile($base.'css/ckeditor.css');
			$layout->addJavaScriptFile($base.'ckeditor/ckeditor.js');
			$layout->addJavaScriptFile($base.'js/ckeditor.min.js');
		}
		else {
			// do not use the class name directly so this file is still PHP 5.2 compatible
			$callback = array('sly\Assets\Util', 'addOnUri');

			$layout->addCSSFile(call_user_func($callback, $me, 'css/ckeditor.css'));
			$layout->addJavaScriptFile(call_user_func($callback, $me, 'ckeditor/ckeditor.js'));
			$layout->addJavaScriptFile(call_user_func($callback, $me, 'js/ckeditor.min.js'));
		}

		$textarea = parent::render();

		if (!$this->clickToLoad) {
			return $textarea;
		}

		$content = $this->getValue();
		$content = trim(strip_tags($content));

		if (!$content) {
			$content = 'Hier klicken, um einen Text einzugeben.';
		}

		$dummy = '<span class="sly-form-read" data-textarea="'.sly_html($textarea).'">'.$content.'<i class="click-to-load"></i></span>';
		$value = '<input type="hidden" name="'.$this->getName().'" value="'.sly_html($this->getValue()).'" />';

		return $value.' '.$dummy;
	}

	/**
	 * Walk through a configuration array and call all callbacks
	 *
	 * @param  mixed $array  configuration structure
	 * @return mixed         the input value with the replaced callback return values
	 */
	protected function applyCallbacks($array) {
		if (!is_array($array)) {
			return $array;
		}

		$ret = array();

		foreach ($array as $key => $value) {
			if (is_array($value) && isset($value['@callback'])) {
				$callback = $value['@callback'];
				$params   = array_key_exists('@params', $value) ? $value['@params'] : array();

				// in 0.8+, you may specify a service instead of a plain, static callback
				if (isset($value['@service'])) {
					$service  = sly_Core::getContainer()->get($value['@service']);
					$callback = array($service, $callback);
				}

				$ret[$key] = call_user_func_array($callback, $params);
			}
			else {
				$ret[$key] = $this->applyCallbacks($value);
			}
		}

		return $ret;
	}
}
