<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

/**
 * Number datatype, using the number HTML5 input type if possible
 */
class WV_Datatype_Number extends WV_Datatype_Base implements WV_IDatatype {
	public function renderForm(sly_Form $form, WV_IProperty $property, $value) {
		$value  = $this->decodeFormValue($property, $value);
		$params = $this->deserializeParams($property->getParams());

		$input = new sly_Form_Input_Number('property_'.$property->getID().'_value', $property->getTitle(), $value);
		$input->setMultilingual($property->isMultilingual());
		$input->setAnnotation(sly_translate($params['annotation'], false));

		if ($params['style']) {
			$input->addStyle($params['style']);
		}

		// setBounds(), setMin() and setMax() do not allow unsetting with null
		// values, so we must be careful what methods to call

		if ($params['min'] !== null)  $input->setMin($params['min']);
		if ($params['max'] !== null)  $input->setMax($params['max']);
		if ($params['step'] !== null) $input->setStep($params['step']);

		WV_Datatype::setHelpTextOnElement($form, $input, $property);
		$form->add($input);

		return $form;
	}

	public function serializeFormFilter($list, WV_IProperty $property) {
		$val_fun = 'intval';
		$params = $this->deserializeParams($property->getParams());
		$allow_null = array_key_exists('allow_null', $params) && $params['allow_null'];
		$allow_float = array_key_exists('allow_float', $params) && $params['allow_float'];
		$min = array_key_exists('min', $params) ? $params['min'] : null;
		$max = array_key_exists('max', $params) ? $params['max'] : null;

		if ($allow_float) {
			$val_fun = array(__CLASS__, 'floatval');
		}

		foreach ($list as &$value) {
			if (strlen(trim($value)) === 0) {
				if ($allow_null) {
					$value = null;
				} else {
					throw new WV_Exception_Datatype(t('devutils_datatype_number_null'));
				}
			} else {
				$value = call_user_func($val_fun, $value);
			}

			if ($value !== null && is_numeric($min) && $value < $min) {
				throw new WV_Exception_Datatype(t('devutils_datatype_number_min', $min));
			}
			if ($value !== null && is_numeric($max) && $value > $max) {
				throw new WV_Exception_Datatype(t('devutils_datatype_number_max', $max));
			}
		}
		return $list;
	}

	public function floatval($val) {
		return floatval(str_replace(',', '.', $val));
	}

	public function deserializeValue($value, array $params) {
		if (($value === null || $value === '') && array_key_exists('allow_null', $params) && $params['allow_null'])
			return null;
		if (array_key_exists('allow_float', $params) && $params['allow_float'])
			return (float) $value;
		return (int) $value;
	}

	protected function getDefaultParams() {
		return array(
			'min'         => null,
			'max'         => null,
			'step'        => null,
			'annotation'  => null,
			'style'       => null,
			'allow_null'  => false,
			'allow_float' => false
		);
	}

	public function isValueEmpty($value, array $params) {
		return ($value === null || $value === '') && (array_key_exists('allow_null', $params) && $params['allow_null']);
	}
}
