<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

define('_WV2_PATH', __DIR__.DIRECTORY_SEPARATOR);

///////////////////////////////////////////////////////////////
// load language DB

$container['sly-i18n']->appendFile(_WV2_PATH.'i18n');

///////////////////////////////////////////////////////////////
// initialize container objects

wv\metainfo\Metainfo::setContainer($container);

// Userland code should always work on the metainfos in the database, never the
// raw YAML based config. So make it obvious and easy to use the correct service.
$container['metainfo-configuration'] = $container->share(function($container) {
	return new wv\metainfo\Configuration\Database($container['sly-registry-persistent']);
});

$container['metainfo-configuration-yaml'] = $container->share(function($container) {
	return new wv\metainfo\Configuration\YAML($container['sly-config']);
});

$container['metainfo-service-property'] = $container->share(function($container) {
	return new wv\metainfo\Property\Service($container['metainfo-configuration-yaml'], $container['metainfo-configuration']);
});

$container['metainfo-property-synchronizer'] = $container->share(function($c) {
	return new wv\metainfo\Property\Synchronizer($c['sly-persistence'], $c['metainfo-service-value']);
});

$container['metainfo-service-value'] = $container->share(function($container) {
	return new wv\metainfo\Value\Service($container['sly-persistence']);
});

$container['metainfo-serializer'] = $container->share(function($container) {
	return new wv\metainfo\Serializer();
});

$container['metainfo-listener-model'] = $container->share(function($container) {
	return new wv\metainfo\Listener\Model();
});

$container['metainfo-listener-backend'] = $container->share(function($container) {
	return new wv\metainfo\Listener\Backend();
});

$container['metainfo-listener-article'] = $container->share(function($container) {
	return new wv\metainfo\Listener\Article();
});

$container['metainfo-listener-category'] = $container->share(function($container) {
	return new wv\metainfo\Listener\Category();
});

$container['metainfo-listener-medium'] = $container->share(function($container) {
	return new wv\metainfo\Listener\Medium();
});

$container['metainfo-listener-user'] = $container->share(function($container) {
	return new wv\metainfo\Listener\User();
});

$container['metainfo-listener-language'] = $container->share(function($container) {
	return new wv\metainfo\Listener\Language();
});

$d = $container->getDispatcher();

///////////////////////////////////////////////////////////////
// extend model API

$cls      = '%metainfo-listener-model%';
$listener = array($cls, 'getObjectMeta');

$d->addListener('SLY_MODEL_ARTICLE_GETMETA',  $listener);
$d->addListener('SLY_MODEL_CATEGORY_GETMETA', $listener);
$d->addListener('SLY_MODEL_MEDIUM_GETMETA',   $listener);
$d->addListener('SLY_MODEL_USER_GETMETA',     $listener);

$listener = array($cls, 'getAllObjectMetaData');

$d->addListener('SLY_MODEL_ARTICLE_GETALLMETADATA',  $listener);
$d->addListener('SLY_MODEL_CATEGORY_GETALLMETADATA', $listener);
$d->addListener('SLY_MODEL_MEDIUM_GETALLMETADATA',   $listener);
$d->addListener('SLY_MODEL_USER_GETALLMETADATA',     $listener);

$cls = '%metainfo-listener-backend%';
$d->addListener('SLY_CACHE_CLEARED', array($cls, 'onClearCache'));

///////////////////////////////////////////////////////////////
// only register service related events when in backend or forced

if (sly_Core::isBackend()) {
	$cls = '%metainfo-listener-backend%';
	// Initialisierungen
	$d->register('SLY_CONTROLLER_FOUND', array($cls, 'init'));
	// add documentation page
	$d->register('DEVUTILS_DOC_SUBPAGES', array($cls, 'addDocPage'));
	// add options to cache select
	$d->addListener('SLY_SYSTEM_CACHES', array($cls, 'onSystemCacheList'));
}

// article API

$cls = '%metainfo-listener-article%';
$d->addListener('SLY_ART_TOUCHED',          array($cls, 'onArticleCloned'));      // ok
$d->addListener('SLY_ART_ADDED',            array($cls, 'onArticleAdded'));       // ok
$d->addListener('SLY_ART_UPDATED',          array($cls, 'onArticleUpdated'));     // ok
$d->addListener('SLY_ART_DELETED',          array($cls, 'onArticleDeleted'));     // ok
$d->addListener('SLY_ART_REVISION_DELETED', array($cls, 'onArticleRevisionDeleted'));     // ok
$d->addListener('SLY_ART_COPIED',           array($cls, 'onArticleCloned'));      // ok
$d->addListener('SLY_ART_TYPE',             array($cls, 'onArticleTypeUpdated')); // ok

$d->addListener('SLY_ART_META_FORM',          array($cls, 'onArticleMetaForm'));       // ok
$d->addListener('SLY_ART_META_FORM_FIELDSET', array($cls, 'onArticleMetaFieldset'));
$d->addListener('SLY_ART_META_UPDATED',       array($cls, 'onArticleMetaUpdated'));    // ok
$d->addListener('SLY_ART_TO_STARTPAGE',       array($cls, 'onArticleToStartarticle')); // ok

// category API

$cls = '%metainfo-listener-category%';
$d->addListener('SLY_CAT_ADDED',   array($cls, 'onCategoryAdded'));
$d->addListener('SLY_CAT_UPDATED', array($cls, 'onCategoryUpdated'));
$d->addListener('SLY_CAT_DELETED', array($cls, 'onCategoryDeleted'));

$d->addListener('SLY_CAT_FORM_EDIT', array($cls, 'onCategoryEditForm'));

// medium API

$cls = '%metainfo-listener-medium%';
$d->addListener('SLY_MEDIA_ADDED',             array($cls, 'onMediumAdded'));
$d->addListener('SLY_MEDIA_UPDATED',           array($cls, 'onMediumUpdated'));
$d->addListener('SLY_MEDIA_DELETED',           array($cls, 'onMediumDeleted'));
$d->addListener('SLY_MEDIA_RESTORED',          array($cls, 'onMediumRestored'));
$d->addListener('SLY_MEDIA_PERMANENT_DELETED', array($cls, 'onMediumPermanentDeleted'));

$d->addListener('SLY_MEDIA_FORM_EDIT', array($cls, 'onMediumEditForm'));
$d->addListener('SLY_MEDIA_USAGES',    array($cls, 'onMediaUsages'));

// user API

$cls = '%metainfo-listener-user%';
$d->addListener('SLY_USER_ADDED',   array($cls, 'onUserAdded'));
$d->addListener('SLY_USER_UPDATED', array($cls, 'onUserUpdated'));
$d->addListener('SLY_USER_DELETED', array($cls, 'onUserDeleted'));

$d->addListener('SLY_USER_FORM',    array($cls, 'onUserForm'));
$d->addListener('SLY_PROFILE_FORM', array($cls, 'onProfileForm'));

// language API

$cls = '%metainfo-listener-language%';
$d->addListener('SLY_CLANG_ADDED',   array($cls, 'onLanguageAdded'));
$d->addListener('SLY_CLANG_DELETED', array($cls, 'onLanguageDeleted'));

// rebuild complete metadata table when importing a dump or clearing the cache
$d->addListener('SLY_DB_IMPORTER_AFTER', array('%metainfo-service-property%', 'rebuild'));

///////////////////////////////////////////////////////////////
// sync metainfos

if (sly_Core::isDeveloperMode()) {
	$d->addListener('SLY_ADDONS_LOADED', function() use ($container) {
		$container['metainfo-service-property']->refresh();
	});
}
