<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

namespace wv\metainfo\Configuration;

use wv\metainfo\Metainfo;
use wv\metainfo\Util;

class Database extends Base {
	protected $registry;

	public function __construct(\sly_Registry_Registry $registry) {
		$this->registry = $registry;
	}

	/**
	 * return array of property configurations
	 *
	 * @param  string $type
	 * @return array
	 */
	public function getAllMetainfoDataForType($type) {
		Metainfo::validateType($type);
		$type = Util::mapTypeToConfig($type);

		return $this->registry->get('wv-metainfo-'.$type, array());
	}

	/**
	 * return property configuration as assoc array
	 *
	 * @param  string $type
	 * @param  string $name
	 * @return array
	 */
	public function getMetainfoData($type, $name) {
		$allData = $this->getAllMetainfoDataForType($type);

		return isset($allData[$name]) ? $allData[$name] : array();
	}

	/**
	 * store current metainfo configuration permanently in the database
	 *
	 * @param string $type
	 * @param array  $data
	 */
	public function setMetainfoData($type, array $data) {
		Metainfo::validateType($type);
		$type = Util::mapTypeToConfig($type);

		$this->registry->set('wv-metainfo-'.$type, $data);
	}
}
