<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

namespace wv\metainfo\Listener;

use sly_Core;
use wv\metainfo\Metainfo;

class Backend {
	/**
	 * Backend initialisieren
	 */
	public function init(array $params) {
		$page   = $params['name'];
		$active = array('devutilsdocs_metainfo', 'structure', 'content', 'contentmeta', 'mediapool', 'user');

		if (in_array($page, $active)) {
			Metainfo::initLayout();
		}
	}

	public function addDocPage(array $params) {
		$params['subject'][] = array('devutilsdocs_metainfo', t('metainfo_metainfos'));

		return $params['subject'];
	}

	protected function showErrors($errors, $type) {
		$flashMessage = sly_Core::getFlashMessage();
		$flashMessage->appendWarning(self::getErrorText($errors, $type));
	}

	protected function getErrorText($errors, $type) {
		$messages  = array(t('metainfo_input_errors'));
		$container = Metainfo::getContainer();
		$service   = $container['metainfo-service-property'];

		foreach ($errors as $metainfo => $message) {
			$metainfo   = $service->findProperty($type, $metainfo);
			$messages[] = sly_translate($metainfo->getTitle(), true).': '.sly_translate($message, true);
		}

		return implode('<br />', $messages);
	}

	protected function getInfosToDisplay(array $available, $assigned, array $required) {
		$return   = array();
		$assigned = sly_makeArray($assigned); // if the object has no metadata yet

		$defaultPos = 1000.001;
		foreach ($available as $name => $info) {
			if (!$info->getBackend()) {
				continue;
			}

		   $value = isset($assigned[$name]) ? $assigned[$name] : $info->getDefault();
			$req   = isset($required[$name]);

			if (!$info->getPosition()) {
				$defaultPos = $defaultPos + 0.001;
			}

			$return[$info->getGroup()][] = array('info' => $info, 'value' => $value, 'required' => $req, 'pos' => !$info->getPosition() ? $defaultPos : $info->getPosition());
		}

		ksort($return, SORT_STRING);

		foreach ($return as $group) {
			usort($group, function(array $a, array $b) {
				return ($a['pos'] < $b['pos']) ? -1 : ($a['pos'] == $b['pos']) ? 0 : 1;
			});
		}

		return $return;
	}

	public function onClearCache($subject, array $params) {
		$container  = Metainfo::getContainer();
		$controller = $container->getApplication()->getCurrentController();
		$isSystem   = is_callable(array($controller, 'isCacheSelected'));
		$clearData  = !$isSystem || $controller->isCacheSelected('wv2-data');
		$rebuild    = !$isSystem || $controller->isCacheSelected('wv2-rebuild');

		if ($clearData) {
			Metainfo::clearDataCache();
		}

		if ($rebuild) {
			$container['metainfo-service-property']->rebuild();
		}

		return $subject;
	}

	/**
	 * SLY_SYSTEM_CACHES
	 *
	 * @param sly_Form_Select_Base $select
	 */
	public function onSystemCacheList(\sly_Form_Select_Base $select) {
		$selected   = $select->getValue();
		$selected[] = 'wv2-data';

		if (sly_Core::isDeveloperMode()) {
			$selected[] = 'wv2-rebuild';
		}

		$select->addValue('wv2-data',    'Metadaten-Cache leeren');
		$select->addValue('wv2-rebuild', 'Metadaten validieren & neu aufbauen');
		$select->setSelected($selected);

		return $select;
	}
}
