<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

abstract class WV42_Events {
	public static function register() {
		$dispatcher = sly_Core::dispatcher();

		if (sly_Core::isBackend()) {
			$dispatcher->addListener('SLY_USER_FORM', array(__CLASS__, 'event_sly_user_form'));
		}

		$dispatcher->addListener('SLY_BACKEND_NAVIGATION_INIT', array(__CLASS__, 'event_sly_backend_navigation_init'));
		$dispatcher->addListener('SLY_USER_ADDED', array(__CLASS__, 'event_sly_user_edited'));
		$dispatcher->addListener('SLY_USER_UPDATED', array(__CLASS__, 'event_sly_user_edited'));
		$dispatcher->addListener('SLY_MODEL_USER_GETROLES', array(__CLASS__, 'event_sly_model_user_getroles'));
		$dispatcher->addListener('SLY_ART_ADDED', array(__CLASS__, 'event_sly_art_added'));
		$dispatcher->addListener('SLY_CAT_ADDED', array(__CLASS__, 'event_sly_art_added'));
		$dispatcher->addListener('SLY_CACHE_CLEARED', array(__CLASS__, 'event_sly_cache_cleared'));
		$dispatcher->addListener('SLY_SEND_RESPONSE', array(__CLASS__, 'event_output_filter'));
		$dispatcher->addListener('SLY_DB_IMPORTER_AFTER', array(__CLASS__, 'event_database_import'));
	}

	public static function event_sly_backend_navigation_init($nav, array $params) {
		$user = $params['user'];

		if ($user && WV42_RBAC::canManageRoles($user)) {
			$page = $nav->get('user', 'system');

			if ($page) {
				// add the user page itself as the first element
				if (count($page->getSubpages()) === 0) {
					$page->addSubpage('user', t('users'));
				}

				$page->addSubpage('rbac', t('rbac'));
			}
			else {
				$nav->addPage('addon', 'rbac', t('rbac'));
			}
		}
	}

	public static function event_sly_model_user_getroles($user) {
		if (!isset($user->roles)) {
			$pacUser     = new pac_User($user->getID());
			$user->roles = $pacUser->getRoles();
		}

		return $user->roles;
	}

	public static function event_sly_user_form($form, $params) {
		$user = $params['user'];

		$roleService = pac_RoleService::getInstance();
		$curRoles    = array();
		$allRoles    = array();

		foreach ($roleService->getAllRoles() as $role) {
			$allRoles[$role->getId()] = sly_translate($role->getTitle());
		}

		if ($user) {
			$pac_user = new pac_User($user->getId());
			$curRoles = $pac_user->getRoles();
		}

		$roleSelect = new sly_Form_Select_DropDown('rbac_user_roles', t('rbac_user_roles'), $curRoles, $allRoles);
		$roleSelect->setMultiple(true);
		$roleSelect->setSize(5);

		$form->addHiddenValue('rbac_form', '1');

		// force the role select to be part of the first fieldset
		$fieldsets = $form->getFieldsets();

		if (!empty($fieldsets)) {
			// find the base fieldset (with sly-num0 as its CSS class)
			$fieldset = null;

			foreach ($fieldsets as $f) {
				if ($f->getID() === 'sly-user-main-fieldset') {
					$fieldset = $f;
					break;
				}
			}

			// just in case someone messed with the classes or we have something < sally 0.9.0-alpha3
			if ($fieldset === null) {
				$fieldset = reset($fieldsets);
			}

			$fieldset->addRow(array($roleSelect));
			$roleSelect->addFormRowClass('rbac-role-select');

			sly_Core::getLayout()->addCSSFile(sly_Util_AddOn::assetBaseUri('webvariants/rbac').'css/backend.less');
		}
		else {
			// in theory, this sould never happen
			$form->addElement($roleSelect);
		}

		return $form;
	}

	public static function event_sly_user_edited($user) {
		if (sly_post('rbac_form', 'boolean', false)) {
			$roles    = sly_postArray('rbac_user_roles', 'int');
			$pac_user = new pac_User($user->getId());
			$pac_user->setRoles($roles);

			WV42_UserCache::delete($user->getId());
		}
	}

	public static function event_sly_art_added($article) {
		/* @var $article sly_Model_Base_Article */
		$articleId = $article->getId();
		$parentId  = $article->getParentId();

		// Because this event handler is called once for each language, we have
		// to make sure we dont't accidentally try to insert the rows more than
		// once.

		$db    = sly_Core::getContainer()->get('sly-persistence');
		$table = pac_Core::ROLES_DESTINATIONS_TABLE;
		$rows  = (int) $db->magicFetch($table, 'COUNT(*)', array(
			'destination' => 'article',
			'value'       => $articleId
		));

		if ($rows === 0) {
			$table     = pac_Core::dbTable($table);
			$statement = pac_Core::prepare(
				'INSERT INTO `'.$table.'` (`destination`, `token`, `role_id`, `value`) '.
				'SELECT `destination`, `token`, `role_id`, :article_id FROM `'.$table.'` WHERE `destination` = :dest AND `value` = :value'
			);

			$statement->execute(array('article_id' => $articleId, 'dest' => 'article', 'value' => $parentId));
		}
	}

	public static function event_sly_cache_cleared() {
		WV42_UserCache::flush();
	}

	public static function event_output_filter($output) {
		WV42_UserCache::persist();

		return $output;
	}

	public static function event_database_import() {
		// remove all user<->role associations that point to users not existing in our local DB.

		$table     = pac_Core::dbTable(pac_Core::ROLES_USERS_TABLE);
		$userTable = pac_Core::dbTable('user');
		$statement = pac_Core::prepare(sprintf('DELETE FROM `%s` WHERE user_id NOT IN (SELECT id FROM `%s`)', $table, $userTable));

		$statement->execute();
	}
}
